#!/usr/bin/perl
#------------------------------------------------------
#ver 1.2

my $line = 10;					#1ページの画像表示数
my $img_path = '../img/';		#画像ディレクトリ
my $home = '/index.html';		#HPアドレス
my $back_img = '';				#背景画像のパス
my $style = [
	'silver',					#バックグラウンドカラー
	'#000000',					#テキストカラー
	'blue',						#リンクカラー
	'purple',					#訪問済みリンクカラー
];

# 設定ここまでです--------------------------------------

#use strict;
#use CGI::Carp qw(fatalsToBrowser);
use CGI;
use utf8;
use open ":utf8";
use Fcntl qw(:DEFAULT :flock);
use Encode qw(from_to decode encode);

my $cgi = CGI->new();
my $rhash = &get_formdata($cgi);

my $info = {
	FORMDATA => $rhash,
	SCRIPT => "$ENV{'SCRIPT_NAME'}\?",
	UA => $ENV{'HTTP_USER_AGENT'},
	IMG_PATH => $img_path,
	MAXLINE => $line,
	HOME_ADDR => $home,
	COLOR => $style,
	BACK_IMG => $back_img,
};

&check_get($rhash);
my $str= &make_list($info);
&display_HTML($info,$str);
exit;

# 画面出力-----------------------------------

sub error{
	my $mes   = shift;
	#$mes = &tag_escape($mes);

	my $html =<<EOL;
<!DOCTYPE html PUBLIC"-//W3C//DTD XHTML1.0 Transitional//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">

<head> 
<title>error</title> 
</head> 
<body>
<h1>error</h1>
<p>
エラーが発生しました
</p>
<p>
$mes
</p>
</body>
</html>
EOL

	utf8::encode($html);
	from_to($html, 'utf8','shiftjis');
	print "Content-type: text/html; charset=shiftjis\n\n";
	print $html;
	exit;
}

sub display_HTML{
	my $info = shift;
	my $str = shift;
	
	my $html = &header($info->{'FORMDATA'}->{'name'},$info->{'BACK_IMG'},$info->{'COLOR'});
	$html .= $str;
	$html .= &footer($info->{'HOME_ADDR'});
	
	utf8::encode($html);
	from_to($html, 'utf8','shiftjis');
	my $byte = length($html);
	
	my $mime = ();
	unless($info->{'UA'} =~ /^DoCoMo\/2/){
		$mime = 'text/html';
	}else{
		$mime = 'application/xhtml+xml';
	}
	
	print "Content-type: $mime; charset=shiftjis\n";
	print 'Content-Length: ', $byte, "\n\n";
	print $html;
	
	return;
}

sub header{
	my ($name,$back_img,$color) = @_;
	my ($back_color,$text,$link,$vlink) = @{$color};
	
	my $html =<<EOL;
<?xml version="1.0" encoding="Shift_JIS"?> 
<!DOCTYPE html PUBLIC "-//OPENWAVE//DTD XHTML Mobile 1.0//EN" 
"http://www.openwave.com/dtd/xhtml-mobile10.dtd"> 
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">

<head> 
<title>$name</title> 
<style type="text/css">
<![CDATA[
a:link{color:$link}
a:visited{color:$vlink}
]]>
</style> 
</head> 
<body style="background-color:$back_color; color:$text; background-image:url($back_img);">
<h1 style="text-align:center;">$name</h1>\n
EOL
	
	return $html;
}

sub footer{
	my $home_addr = shift;
	
	my $html =<<EOL;
<hr />
<p style="text-align:center"><a href="$home_addr" style="-wap-accesskey:2">[2] HP</a></p>

</body>
</html>
EOL
	return $html;
}

# ページ作成---------------------------------------

sub make_list{
	my $info = shift;
	my $rhash = $info->{'FORMDATA'};
	my $max_line = $info->{'MAXLINE'};
	my $path = $info->{'IMG_PATH'};
	my $name = $rhash->{'name'};
	
	if(!opendir(DIR,$info->{'IMG_PATH'}.$name)){
		&error("ディレクトリがありません");
	}

	my @filename= grep(!/^\.+/,readdir DIR);
	close DIR;
	
	@filename = grep(/^.+\.+\w+$/,@filename);
	
			
	my $all = @filename;
	unless($all){return "<p>ファイルがありません</p>";}
	
	my $max_page  = int($all / $max_line);
	$max_page++ if $all % $max_line;
	
	my $this_page = $rhash->{'this_page'} or 1;
	my $which     = $rhash->{'which'}     || "";

	if($which eq 'next'){
		$this_page++;
	}elsif($which eq 'before'){
		$this_page--;
	}
	
	$this_page = 1  if $this_page < 1;
	$this_page = $max_page  if $this_page > $max_page;

	
	my $start = ($this_page -1) * $max_line;
	my $end = ();
	if ($this_page == $max_page){
		unless ($all % $max_line){
			$end = $this_page * $max_line -1;
		}else{
			$end = ($this_page * $max_line -1) - ($max_line - $all % $max_line);
		}
	}else{
		$end = $this_page * $max_line -1;
	}
	
	my $string = ();
	foreach($start .. $end){
		my $uri = qq|$info->{'IMG_PATH'}$rhash->{'name'}/$filename[$_]|;
		my $size = (stat $uri)[7];
		if($size >= 1000){
			$size = $size / 1000;
			$size = sprintf("%.1f",$size) . ' KB';
		}else{
			$size .= ' byte';
		}
		my $type;
		if($filename[$_] =~ /(\.\w+$)/){$type = $1;}
		
		$string .= qq|<a href="$uri" style="text-decoration:none">file|.($_ + 1).qq|$type </a>($size)<br />\n|;
	}
		
	$name = tag_escape($name);
	my $page_link = &make_page_link($info,$this_page,$max_page,$name);
	
	my $html =<<EOL;

<p style="margin:1em 0 1em 1em; text-align:left">\n
$string
</p>\n\n
$page_link\n
<!-- cotatu.script cotatu.undo.jp/-->\n
EOL

	return $html;
}

sub make_page_link{
	my $info = shift;
	my $this_page = shift;
	my $max_page = shift;
	my $name = shift;
	
	my $befor_link;
	if($this_page > 1){
		my $uri = qq|$info->{'SCRIPT'}&amp;name=$name&amp;this_page=$this_page&amp;which=before|;
		$befor_link = qq|<a href="$uri" style="-wap-accesskey:1">[1] 前へ</a>\n|;
	}else{
		$befor_link = '前へ';
	}
	
	my $next_link;
	if($this_page < $max_page){
		my $uri = qq|$info->{'SCRIPT'}&amp;name=$name&amp;this_page=$this_page&amp;which=next|;
		$next_link = qq|<a href="$uri" style="-wap-accesskey:3">[3] 次へ</a>|;
	}else{
		$next_link = '次へ';
	}
	
	my $page_num_link;
	if($max_page > 1){
		for my $i (1.. $max_page){
			if($i eq $this_page){
				$page_num_link .= "$i \n";
			}else{
				my $uri = qq|$info->{'SCRIPT'}&amp;name=$name&amp;this_page=$i|;
				$page_num_link .= qq|<a href="$uri">$i</a> \n|;
			}
		}
	}
	
	my $html =<<EOL;
<p style="text-align:center">
$befor_link　$next_link
</p>

<p style="text-align:center">
$page_num_link
</p>

EOL
	return $html;
}

# チェック関係-------------------------------
sub tag_escape{
	my $string = shift;
	my $uriflag = shift; #0はノーマル。1はhttpリンク作成。2は$wordにリンク作成
	my $word = shift;
	
	my %escape = (
		'<' => "&lt;",
		'>' => "&gt;",
		'&' => "&amp;",
		'"' => "&quot;",
		"'" => "&#39;",
		);
	
	if(!$uriflag){
		$string =~ s/([<>&\"\'])/$escape{$1}/g;
		return $string;
	}
	
	if($uriflag == 1){
		$string =~ s{([<>&\"\']|(?:(?:http|https|ftp|mailto):[0-9A-Za-z\;\/\?\:\@\&\=\+\$\,\[\]\-\_\.\!\~\*\'\(\)\%]+))}{length($1) == 1 ? $escape{$1} : qq(<a href="$1">$1</a>);}exg;
		return $string;
	}
	
	if($uriflag == 2){
		$string =~ s{([<>&\"\']|(?:(?:http|https|ftp|mailto):[0-9A-Za-z\;\/\?\:\@\&\=\+\$\,\[\]\-\_\.\!\~\*\'\(\)\%]+))}{length($1) == 1 ? $escape{$1} : qq(<a href="$1">$word</a>);}exg;
		return $string;
	}
}

sub check_get{
	my $rhash = shift;
	
	unless($rhash->{'name'}){&error('nameがありません');}

	if($rhash->{'which'} and !($rhash->{'which'} eq 'before'||$rhash->{'which'} eq 'next')){
		&error('不正な操作です');
	}
	if($rhash->{'num'} and $rhash->{'num'} =~ /[^0-9]/ ){
		&error('不正な操作です');
	}
	
	return;
}

#---------------------------------------------
sub get_formdata{
	my $cgi = shift;
	my @hashkey = $cgi->param;
	
	my %hash;
	foreach my $key(@hashkey){
		$hash{$key} = decode('shiftjis', $cgi->param($key));
		next if $hash{$key} =~ /^\s*$/;
		
		$hash{$key} =~ s/\t//g;
		$hash{$key} =~ s/\x{000D}\x{000A}/\n/g;
		$hash{$key} =~ tr/\x{000D}\x{000A}/\n\n/;
		$hash{$key} =~ s/^\s+//;
		$hash{$key} =~ s/\s+$//;
	}
	return \%hash;
}

__END__